@@ -4,4 +4,8 @@
 #define DEFAULT_HIDE_ON_STARTUP 0
 #define DEFAULT_TRAY_ICON_ENABLE 1
 #define DEFAULT_AUTOCONNECT 0
-#define DEFAULT_TRAY_MARKUP "[<span size=\"small\">%name%</span>\n][<span size=\"large\">%title%</span>\n][%artist%][\n<span size=\"small\">%album% [(track %track%)]</span>]|%shortfile%|"
+#define DEFAULT_TRAY_MARKUP "[<span size=\"small\">%name%</span>\n][<span size=\"large\" weight=\"bold\">%title%</span>\n][%artist%][\n<span size=\"small\">%album% [(track %track%)]</span>]|%shortfile%|"
+
+
+#define DEFAULT_PLAYLIST_PLAYER_MARKUP "[<span size=\"small\">%name%</span>\n][<span size=\"large\" weight=\"bold\">%title%</span>\n][%artist%][\n<span size=\"small\">%album%</span>]|%shortfile%|"
+
