@@ -17,6 +17,8 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+#include <config.h>
+#ifdef ENABLE_TRAYICON
 
 #include <string.h>
 #include <libintl.h>
@@ -470,3 +472,5 @@ egg_tray_icon_get_orientation (EggTrayIcon *icon)
 
   return icon->orientation;
 }
+
+#endif
