@@ -1,5 +1,5 @@
 /*
- *Copyright (C) 2004 Qball Cow <Qball@qballcow.nl>
+ * Copyright (C) 2004-2006 Qball Cow <Qball@qballcow.nl>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,6 +21,7 @@
 #include <gtk/gtk.h>
 #include <string.h>
 #include <glade/glade.h>
+#include <config.h>
 #include "main.h"
 
 GladeXML *xml_id3_window = NULL;
@@ -59,9 +60,9 @@ void id3_status_update()
 	if(songs == NULL) return;
 	song =  songs->data;
 	if(song == NULL) return;
-	if(song->id == mpd_ob_player_get_current_song_id(connection) && song->id != MPD_SONG_NO_ID)
+	if(song->id == mpd_player_get_current_song_id(connection) && song->id != MPD_SONG_NO_ID)
 	{
-		char *temp = g_strdup_printf("%i kb/s", mpd_ob_status_get_bitrate(connection));
+		char *temp = g_strdup_printf("%i kbps", mpd_status_get_bitrate(connection));
 		gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(xml_id3_window,"bitrate_label")),temp);
 		g_free(temp);
 
@@ -75,11 +76,11 @@ void id3_status_update()
 		gint min = (int) (song->time / 60);
 		gint sec = song->time - min * 60;
 		gchar *buf1 =NULL;
-		if(song->id == mpd_ob_player_get_current_song_id(connection))
+		if(song->id == mpd_player_get_current_song_id(connection))
 		{
 			buf1 = g_strdup_printf ("%02i:%02i/%02i:%02i",
-					mpd_ob_status_get_elapsed_song_time(connection)/60,
-					mpd_ob_status_get_elapsed_song_time(connection)%60,
+					mpd_status_get_elapsed_song_time(connection)/60,
+					mpd_status_get_elapsed_song_time(connection)%60,
 					min, sec);
 		}
 		else
@@ -100,14 +101,16 @@ void id3_status_update()
 
 void create_window (int song)
 {
-	mpd_Song *songstr = mpd_ob_playlist_get_song(connection, song);
+	char *path;
+	mpd_Song *songstr = mpd_playlist_get_song(connection, song);
+
 	if(songstr == NULL)
 	{
 		return;
 	}
-	
-	xml_id3_window = glade_xml_new (GLADE_PATH "gmpc.glade", "id3_info_window", NULL);
-
+	path = gmpc_get_full_glade_path("gmpc.glade");	
+	xml_id3_window = glade_xml_new (path,"id3_info_window", NULL);
+	g_free(path);
 	/* check for errors and axit when there is no gui file */
 	if (xml_id3_window == NULL)
 	{
@@ -118,6 +121,52 @@ void create_window (int song)
 	songs = g_list_append (songs, songstr);
 	set_text (songs);
 }
+void id3_cover_art_fetched(mpd_Song *song)
+{
+	mpd_Song *current = NULL;
+	if(songs == NULL) return;
+	if(songs->data== NULL) return;
+	current = songs->data;
+	if(current->artist && current->album)
+	{
+		if(!strcmp(current->artist,song->artist) &&
+				!strcmp(current->album, song->album))
+		{
+			set_text(songs);	
+		}
+	}
+}
+
+void id3_cover_art_clicked()
+{
+	mpd_Song *song;
+	if(songs == NULL)  return;
+	if(songs->data == NULL) return;
+	song = songs->data;
+	if(song->artist && song->album)
+	{
+		if(cover_art_edit_cover(song->artist, song->album))
+		{
+			GdkPixbuf *pb = NULL;
+			gchar *path = NULL;
+			int ret = cover_art_fetch_image_path(song, &path);
+			if(ret == COVER_ART_OK_LOCAL){
+				pb = gdk_pixbuf_new_from_file_at_size(path,300,300,NULL);                                    		
+				gtk_image_set_from_pixbuf(GTK_IMAGE(glade_xml_get_widget(xml_id3_window, "cover_image")),pb);
+				gtk_widget_show(glade_xml_get_widget(xml_id3_window, "cover_event"));
+				g_object_unref(pb);           
+			}
+			else{
+				gtk_widget_hide(glade_xml_get_widget(xml_id3_window, "cover_event"));
+			}			
+			if(path) g_free(path);
+		}                                                                                                    		
+
+
+
+	}
+}
+
 
 void set_text (GList * node)
 {
@@ -174,21 +223,21 @@ void set_text (GList * node)
 		gtk_label_set_text (GTK_LABEL(glade_xml_get_widget(xml_id3_window, "track_label")), "");
 	}
 	if (song->genre != NULL)
-        {
-        	gtk_label_set_text (GTK_LABEL(glade_xml_get_widget(xml_id3_window, "genre_label")), song->genre);
-        }
-        else
-        {
-        	gtk_label_set_text (GTK_LABEL(glade_xml_get_widget(xml_id3_window, "genre_label")), "");
-        }
+	{
+		gtk_label_set_text (GTK_LABEL(glade_xml_get_widget(xml_id3_window, "genre_label")), song->genre);
+	}
+	else
+	{
+		gtk_label_set_text (GTK_LABEL(glade_xml_get_widget(xml_id3_window, "genre_label")), "");
+	}
 	if (song->composer != NULL)
-        {
-        	gtk_label_set_text (GTK_LABEL(glade_xml_get_widget(xml_id3_window, "composer_label")), song->composer);
-        }
-        else
-        {
-        	gtk_label_set_text (GTK_LABEL(glade_xml_get_widget(xml_id3_window, "composer_label")), "");
-        }
+	{
+		gtk_label_set_text (GTK_LABEL(glade_xml_get_widget(xml_id3_window, "composer_label")), song->composer);
+	}
+	else
+	{
+		gtk_label_set_text (GTK_LABEL(glade_xml_get_widget(xml_id3_window, "composer_label")), "");
+	}
 	if (song->file != NULL)
 	{
 		gchar *buf1 = g_path_get_basename (song->file);
@@ -210,9 +259,9 @@ void set_text (GList * node)
 		gtk_label_set_text (GTK_LABEL(glade_xml_get_widget(xml_id3_window, "path_label")), "");
 	}
 
-	if(song->id == mpd_ob_player_get_current_song_id(connection) && song->id != MPD_SONG_NO_ID)
+	if(song->id == mpd_player_get_current_song_id(connection) && song->id != MPD_SONG_NO_ID)
 	{
-		char *temp = g_strdup_printf("%i kbps", mpd_ob_status_get_bitrate(connection));
+		char *temp = g_strdup_printf("%i kbps", mpd_status_get_bitrate(connection));
 		gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(xml_id3_window,"bitrate_label")),temp);
 		g_free(temp);
 
@@ -252,6 +301,29 @@ void set_text (GList * node)
 	{
 		gtk_widget_set_sensitive (glade_xml_get_widget(xml_id3_window, "button_next"), TRUE);
 	}
+	if(song){
+		gchar *path= NULL;
+		int ret = 0;
+		ret = cover_art_fetch_image_path(song, &path);
+		if(ret == COVER_ART_OK_LOCAL)
+		{
+			GdkPixbuf *pb = NULL;
+			pb = gdk_pixbuf_new_from_file_at_size(path,300,300,NULL);
+			gtk_image_set_from_pixbuf(GTK_IMAGE(glade_xml_get_widget(xml_id3_window, "cover_image")),pb);
+			gtk_widget_show(glade_xml_get_widget(xml_id3_window, "cover_event"));
+			g_object_unref(pb);
+		}
+		else{
+			gtk_widget_hide(glade_xml_get_widget(xml_id3_window, "cover_event"));
+		}
+		if(path) g_free(path);
+		if(ret == COVER_ART_NOT_FETCHED)
+		{
+			cover_art_fetch_image(song,
+					(CoverArtCallback)id3_cover_art_fetched,NULL);
+		}
+
+	}
 }
 
 void id3_next_song ()
@@ -276,7 +348,9 @@ void call_id3_window_song(mpd_Song *songstr)
 	}
 	if(xml_id3_window == NULL)
 	{
-		xml_id3_window = glade_xml_new (GLADE_PATH "gmpc.glade", "id3_info_window", NULL);
+		char *path = gmpc_get_full_glade_path("gmpc.glade");
+		xml_id3_window = glade_xml_new (path, "id3_info_window", NULL);
+		g_free(path);
 
 		/* check for errors and axit when there is no gui file */
 		if (xml_id3_window == NULL)
@@ -303,7 +377,7 @@ void call_id3_window (int song)
 	}
 	else
 	{
-		mpd_Song  *songstr = mpd_ob_playlist_get_song(connection, song);
+		mpd_Song  *songstr = mpd_playlist_get_song(connection, song);
 		if(songstr != NULL)
 		{
 			songs = g_list_append (songs,songstr);
