@@ -1,5 +1,5 @@
 /*
- *Copyright (C) 2004 Qball Cow <Qball@qballcow.nl>
+ *Copyright (C) 2004-2006 Qball Cow <Qball@qballcow.nl>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,22 +24,40 @@
 #include <glade/glade.h>
 #include <stdlib.h>
 #include <gdk/gdkkeysyms.h>
+#include <config.h>
 #include "main.h"
 #include "misc.h"
-#include "strfsong.h"
 #include "playlist3.h"
 #include "config1.h"
-extern config_obj *config;
 
+
+/* player.c*/
+/* scrolling title in main window */
+typedef struct	{
+	char *msg;
+	char *base_msg;
+	char *popup_msg;
+	int pos;
+	int up;
+	gboolean do_scroll;
+	gboolean exposed;
+} scrollname;
+
+GtkAllocation player_wsize = {0,0,0,0};
+GladeXML *xml_main_window = NULL;
+void player_song_changed();
+void player_state_changed(int state);
+void volume_change_update();
 #define TITLE_LENGTH 42
 gint DISPLAY_WIDTH = 240;
-
-scrollname scroll = {NULL, NULL, NULL,NULL, 0,0, TRUE,TRUE};
+guint msg_timeout = 0;
+scrollname scroll = {NULL, NULL, NULL,/*NULL,*/ 0,0, TRUE,TRUE};
 PangoLayout *layout = NULL, *time_layout = NULL;
 guint expose_display_id = 0;
 
 guint seek = FALSE;
 guint volume = FALSE;
+gboolean player_hidden = FALSE;
 
 void title_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 {
@@ -55,11 +73,11 @@ void time_exposed(GtkWidget *window)
 	height = height/PANGO_SCALE;
 	gtk_widget_size_request(window, &req);
 
-	
+
 	if(height+6 != req.height || width+6 > req.width)
 	{
-		gtk_widget_set_size_request(window,width+6,height+6);	
-	}                                                       	
+		gtk_widget_set_size_request(window,width+6,height+6);
+	}
 
 	gdk_draw_rectangle(GDK_DRAWABLE(window->window),
 			window->style->base_gc[GTK_STATE_NORMAL],
@@ -100,7 +118,7 @@ void display_exposed(GtkWidget *window)
 			window->style->base_gc[GTK_STATE_NORMAL],
 			TRUE,
 			0,0,
-			DISPLAY_WIDTH,req.height);                            
+			DISPLAY_WIDTH,req.height); 
 	if(width <= DISPLAY_WIDTH-5)
 	{
 		gdk_draw_layout(GDK_DRAWABLE(window->window), 
@@ -137,31 +155,29 @@ void display_exposed(GtkWidget *window)
 
 		}
 	}
-
 	gdk_draw_rectangle(GDK_DRAWABLE(window->window),
 			window->style->dark_gc[GTK_STATE_NORMAL],
 			FALSE,
 			0,0,
-			DISPLAY_WIDTH-1,req.height-1);                              
+			DISPLAY_WIDTH-1,req.height-1);
 	g_signal_handler_unblock(G_OBJECT(window), expose_display_id);
-}	
+}
 gboolean update_msg()
 {
 	int width;
+	if(!xml_main_window) return TRUE;
 	/* scroll will be -1 when there is getting stuff updated. hopefully this fixes the nasty segfault in pango*/
 	if(scroll.exposed)
 	{
 		if(scroll.msg != NULL) g_free(scroll.msg);
 		/* set the correct message in the msg box. and set posistion on 0 */
-		if(scroll.popup_msg != NULL)
-		{
+		if(scroll.popup_msg != NULL) {
 			scroll.msg = g_strdup(scroll.popup_msg);
-		}	
-		else if(scroll.base_msg != NULL)
-		{
+		} else if(scroll.base_msg != NULL){
 			scroll.msg = g_strdup(scroll.base_msg);
+		} else {
+			scroll.msg = g_strdup(_("Gnome Music Player Client"));
 		}
-		else scroll.msg = g_strdup(_("Gnome Music Player Client"));
 		scroll.pos = 0;
 		scroll.up = 0;
 		pango_layout_set_text(layout, scroll.msg, -1);
@@ -187,14 +203,12 @@ gboolean update_msg()
 		{
 			if(scroll.do_scroll)
 			{
-				scroll.pos+=4;
+				scroll.pos+=3;
 			}
 			else
 			{
 				scroll.pos = -2;
-
 			}
-
 		}
 		gtk_widget_queue_draw(window);
 
@@ -209,7 +223,7 @@ void msg_set_base(gchar *msg)
 	int i =0;
 	if(msg == NULL) return;
 	/* don't update when its the same string :) */
-	scroll.do_scroll = TRUE;
+
 	if(scroll.msg != NULL)
 	{
 		if(!strcmp(scroll.msg, msg)) return;
@@ -219,11 +233,14 @@ void msg_set_base(gchar *msg)
 		g_free(scroll.base_msg);
 		scroll.base_msg = NULL;
 	}
-	if(!g_utf8_validate(msg, -1, NULL))
-	{
+	if(!g_utf8_validate(msg, -1, NULL)){
 		scroll.base_msg = g_strdup(_("No valid UTF-8. Please check your locale"));
 	}
-	else	scroll.base_msg = g_strdup(msg);
+	else {
+		scroll.base_msg = g_strdup(msg);
+	}
+
+	scroll.do_scroll = TRUE;
 	for(;msg[i] != '\0';i++)
 	{
 		if(msg[i] == '\n')
@@ -238,34 +255,44 @@ void msg_set_base(gchar *msg)
 void msg_push_popup(gchar *msg)
 {
 	if(msg == NULL) return;
-	if(scroll.queue == NULL)
+	if(msg_timeout)
 	{
-		scroll.queue = g_queue_new();
+		g_source_remove(msg_timeout);
+		msg_timeout = 0;
 	}
+
 	if(!g_utf8_validate(msg, -1, NULL))
 	{
-		g_queue_push_tail(scroll.queue,g_strdup(_("No valid UTF-8. Please check your locale")));
-		scroll.popup_msg = g_queue_peek_tail(scroll.queue);
+		if(scroll.popup_msg)g_free(scroll.popup_msg);
+		scroll.popup_msg = g_strdup(_("No valid UTF-8. Please check your locale"));
 	}
 	else
 	{
-		g_queue_push_tail(scroll.queue,g_strdup(msg));
-		scroll.popup_msg = g_queue_peek_tail(scroll.queue);
+		if(scroll.popup_msg)g_free(scroll.popup_msg);
+		scroll.popup_msg =g_strdup(msg);
 	}
 	scroll.exposed = TRUE;
 }
-
-void msg_pop_popup()
+void msg_push_popup_timeout(gchar *msg, int timeout)
 {
+	msg_push_popup(msg);
+	msg_timeout = g_timeout_add(timeout, (GSourceFunc)msg_pop_popup, NULL);
+}
+
+int msg_pop_popup()
+{
+	if(msg_timeout)
+	{
+		g_source_remove(msg_timeout);
+		msg_timeout = 0;
+	}
 	if(scroll.popup_msg != NULL)
 	{
-		char *msg = g_queue_peek_tail(scroll.queue);
-
-		g_queue_pop_tail(scroll.queue);
-		scroll.popup_msg = g_queue_peek_tail(scroll.queue);
-		g_free(msg);
+		g_free(scroll.popup_msg);
+		scroll.popup_msg = NULL;
 	}
 	scroll.exposed = TRUE;
+	return FALSE;
 }
 
 /* this updates the player.. this is called from the update function */
@@ -273,18 +300,12 @@ void msg_pop_popup()
 
 int update_player()
 {
-	if(!mpd_ob_check_connected(connection)) return FALSE;
-	/* update the volume slider */
-	GtkRange *scale = (GtkRange *)glade_xml_get_widget(xml_main_window, "volume_slider");
-	if((int)gtk_range_get_value(scale) != mpd_ob_status_get_volume(connection))
-	{
-		gtk_range_set_value(scale, (double)mpd_ob_status_get_volume(connection));
-	}    
-
+	if(!xml_main_window) return FALSE;
+	if(!mpd_check_connected(connection)) return FALSE;
 	/* things that only need to be updated during playing */
 	{
-		int totalTime = mpd_ob_status_get_total_song_time(connection);
-		int elapsedTime = mpd_ob_status_get_elapsed_song_time(connection);		
+		int totalTime = mpd_status_get_total_song_time(connection);
+		int elapsedTime = mpd_status_get_elapsed_song_time(connection);		
 		/* update the progress bar */
 		if(!seek){
 			GtkRange *scale = (GtkRange *)glade_xml_get_widget(xml_main_window, "progress_slider");
@@ -307,17 +328,17 @@ int update_player()
 				{
 					e_sec = (e_min % 60);
 					e_min = (int)(e_min/60);
-				}                                				
+				}
 				buf = g_strdup_printf("%02i:%02i", abs(e_min), abs(e_sec));
 			}
 			else if (cfg_get_single_value_as_int(config, "player", "time-format") == TIME_FORMAT_REMAINING)
 			{
 				/* if more then 100 minutes player change to hh:mm */
 				if(abs(r_min) >= 100)
-				{                                                               				
+				{
 					r_sec = (r_min % 60);
 					r_min = (int)(r_min/60);
-				}                                								
+				}
 				buf = g_strdup_printf("-%02i:%02i", abs(r_min), abs(r_sec));
 			}
 			else{
@@ -336,25 +357,91 @@ int update_player()
 			g_free(buf);
 		}
 	}
-
-	/* update random and repeat button */
-	/* lock it to stop them from toggling and triggering another toggle*/
-	if(mpd_ob_player_get_repeat(connection) != gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml_main_window, "rep_button"))))
-	{
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml_main_window, "rep_button")), mpd_ob_player_get_repeat(connection));
-	}
-	if(mpd_ob_player_get_random(connection) != gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml_main_window, "rand_button"))))
-	{
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml_main_window, "rand_button")), mpd_ob_player_get_random(connection));
-	}
 	return FALSE;
 }
 
-
-void player_state_changed(int old_state, int state)
+void player_mpd_state_changed(MpdObj *mi, ChangedStatusType what, void *userdata)
 {
-	GtkWidget *image = glade_xml_get_widget(xml_main_window, "play_button_image");
-	if(state == MPD_OB_PLAYER_STOP || state == MPD_OB_PLAYER_UNKNOWN)
+	gchar *msg = NULL;
+	if(xml_main_window == NULL)return;
+	if(what&MPD_CST_RANDOM)
+	{
+		if(mpd_player_get_random(connection) != 
+				gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml_main_window, "rand_button"))))
+		{
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml_main_window, "rand_button")),
+				       mpd_player_get_random(connection));
+		}
+		msg = g_strdup_printf("%s: %s", _("Random"), (mpd_player_get_random(connection))?_("On"):_("Off"));
+		msg_push_popup_timeout(msg, 2000);
+		g_free(msg);
+	}
+	if(what&MPD_CST_REPEAT)
+	{
+		if(mpd_player_get_repeat(connection) !=
+				gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml_main_window, "rep_button"))))
+		{
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml_main_window, "rep_button")),
+				       mpd_player_get_repeat(connection));
+		}
+		msg = g_strdup_printf("%s: %s", _("Repeat"), (mpd_player_get_repeat(connection))?_("On"):_("Off"));
+		msg_push_popup_timeout(msg, 2000);
+		g_free(msg);
+
+	}
+	if(what&MPD_CST_VOLUME)
+	{
+		GtkRange *scale = (GtkRange *)glade_xml_get_widget(xml_main_window, "volume_slider");
+		if((int)gtk_range_get_value(scale) != mpd_status_get_volume(connection))
+		{
+			gtk_range_set_value(scale, (double)mpd_status_get_volume(connection));
+		}
+		msg = g_strdup_printf("%s: %3i%%", _("Volume"), mpd_status_get_volume(connection));
+		msg_push_popup_timeout(msg, 2000);
+		g_free(msg);
+	}
+	if(what&MPD_CST_CROSSFADE)
+	{
+		if(mpd_status_get_crossfade(connection) >0)
+		{
+			msg = g_strdup_printf("%s: %i sec", _("Crossfade"), mpd_status_get_crossfade(connection));
+		}
+		else
+		{
+			msg = g_strdup_printf("%s: %s", _("Crossfade"), _("Off"));
+		}
+		msg_push_popup_timeout(msg, 2000);
+		g_free(msg);
+	}
+	if(what&MPD_CST_SONGID)
+	{
+		/* we want to see the new song */
+		msg_pop_popup();
+		player_song_changed();
+	}
+	if(what&MPD_CST_STATE)
+	{
+		/* we want to see the new song */
+		msg_pop_popup();
+		player_state_changed(mpd_player_get_state(connection));
+	}
+	if(what&MPD_CST_TOTAL_TIME)
+	{
+		update_player();
+	}
+	if(what&MPD_CST_ELAPSED_TIME)
+	{
+		update_player();
+	}
+}
+
+void player_state_changed(int state)
+{
+	
+	GtkWidget *image = NULL;
+	if(!xml_main_window) return;
+	image = glade_xml_get_widget(xml_main_window, "play_button_image");
+	if(state == MPD_PLAYER_STOP || state == MPD_PLAYER_UNKNOWN)
 	{
 		GtkWidget *entry;
 		msg_set_base(_("GMPC - Stopped"));
@@ -367,7 +454,7 @@ void player_state_changed(int old_state, int state)
 			pango_layout_set_text(time_layout, "-00:00", -1);
 
 		}
-		else	
+		else
 		{
 			pango_layout_set_text(time_layout, "0.0 %", -1);
 		}
@@ -376,8 +463,8 @@ void player_state_changed(int old_state, int state)
 		gtk_range_set_value(GTK_RANGE(entry), 0);
 
 	}
-	if(state == MPD_OB_PLAYER_PLAY)
-	{	
+	if(state == MPD_PLAYER_PLAY)
+	{
 		gtk_image_set_from_stock(GTK_IMAGE(image),"gtk-media-pause", GTK_ICON_SIZE_BUTTON);
 	}
 	else
@@ -385,19 +472,20 @@ void player_state_changed(int old_state, int state)
 		gtk_image_set_from_stock(GTK_IMAGE(image), "gtk-media-play", GTK_ICON_SIZE_BUTTON);
 	}
 
-	if(state != MPD_OB_PLAYER_PLAY && state != MPD_OB_PLAYER_PAUSE)
+	if(state != MPD_PLAYER_PLAY && state != MPD_PLAYER_PAUSE)
 	{
 		msg_set_base(_("Gnome Music Player Client"));
 		if(cfg_get_single_value_as_int_with_default(config, "player", "window-title",TRUE))
 		{
-			gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget(xml_main_window, "main_window")), _("Gnome Music Player Client"));	
+			gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget(xml_main_window, "main_window")),
+					_("Gnome Music Player Client"));
 		}
 	}
 	else
 	{
 		gchar buffer[1024];
 		char *string =cfg_get_single_value_as_string_with_default(config, "player","display_markup",DEFAULT_PLAYER_MARKUP);
-		strfsong(buffer, 1024,string,mpd_ob_playlist_get_current_song(connection));
+		mpd_song_markup(buffer, 1024,string,mpd_playlist_get_current_song(connection));
 		cfg_free_string(string);
 		msg_set_base(buffer);
 		if(cfg_get_single_value_as_int_with_default(config, "player", "window-title",TRUE))
@@ -412,7 +500,7 @@ void player_state_changed(int old_state, int state)
 			}
 			/* malloc a new string.. size of old string + the number of needed "stripes"*2 ('\n' -> ' - ' == +2 chars) + a ending \0*/
 			new_buf = (char *)g_malloc0((i+2*stripes+1)*sizeof(char ));
-			
+
 			for(i=0;buffer[i] != '\0';i++)
 			{
 
@@ -429,7 +517,7 @@ void player_state_changed(int old_state, int state)
 				}
 				j++;
 			}
-			
+
 
 			gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget(xml_main_window, "main_window")), new_buf);	
 			g_free(new_buf);
@@ -439,11 +527,11 @@ void player_state_changed(int old_state, int state)
 }
 
 
-void player_song_changed(int oldsong, int newsong)
+void player_song_changed()
 {
-	int state = mpd_ob_player_get_state(connection);
-
-	if(state != MPD_OB_PLAYER_PLAY && state != MPD_OB_PLAYER_PAUSE)
+	int state = mpd_player_get_state(connection);
+	if(!xml_main_window) return;
+	if(state != MPD_PLAYER_PLAY && state != MPD_PLAYER_PAUSE)
 	{
 		msg_set_base(_("Gnome Music Player Client"));
 		if(cfg_get_single_value_as_int_with_default(config, "player", "window-title",TRUE))
@@ -455,7 +543,7 @@ void player_song_changed(int oldsong, int newsong)
 	{
 		gchar buffer[1024];
 		char *string = cfg_get_single_value_as_string_with_default(config, "player","display_markup",DEFAULT_PLAYER_MARKUP);
-		strfsong(buffer, 1024, string,mpd_ob_playlist_get_current_song(connection));
+		mpd_song_markup(buffer, 1024, string,mpd_playlist_get_current_song(connection));
 		cfg_free_string(string);
 		msg_set_base(buffer);
 		if(cfg_get_single_value_as_int_with_default(config, "player", "window-title",TRUE))
@@ -500,8 +588,8 @@ void player_song_changed(int oldsong, int newsong)
 /* show time to seek to in entry box */
 int progress_seek_start()
 {
-	if(mpd_ob_player_get_state(connection) != MPD_OB_PLAYER_PLAY && 
-			mpd_ob_player_get_state(connection) != MPD_OB_PLAYER_PAUSE)
+	if(mpd_player_get_state(connection) != MPD_PLAYER_PLAY && 
+			mpd_player_get_state(connection) != MPD_PLAYER_PAUSE)
 	{
 		return TRUE;
 	}
@@ -514,12 +602,12 @@ void change_progress_update()
 {
 	if(seek)
 	{
-		if(mpd_ob_check_connected(connection))
+		if(mpd_check_connected(connection))
 		{
 			GtkRange *scale = (GtkRange *)glade_xml_get_widget(xml_main_window, "progress_slider");
 			gchar *buf = NULL;
 			gdouble value = gtk_range_get_value(scale);
-			int totaltime = mpd_ob_status_get_total_song_time(connection);
+			int totaltime = mpd_status_get_total_song_time(connection);
 			int newtime = (int)(totaltime*(double)(value/100));
 			if(cfg_get_single_value_as_int_with_default(config, "player", "time-format",TIME_FORMAT_ELAPSED)  == TIME_FORMAT_ELAPSED)
 			{
@@ -553,17 +641,17 @@ int progress_seek_stop()
 {
 	msg_pop_popup();
 	seek = FALSE;
-	if(!mpd_ob_check_connected(connection))
+	if(!mpd_check_connected(connection))
 	{
 		return TRUE;
 	}
-	else if(mpd_ob_player_get_state(connection) == MPD_OB_PLAYER_PLAY || mpd_ob_player_get_state(connection) == MPD_OB_PLAYER_PAUSE)
+	else if(mpd_player_get_state(connection) == MPD_PLAYER_PLAY || mpd_player_get_state(connection) == MPD_PLAYER_PAUSE)
 	{
 		GtkRange *scale = (GtkRange *)glade_xml_get_widget(xml_main_window, "progress_slider");
 		gdouble value = gtk_range_get_value(scale);
-		int change = (int)(mpd_ob_status_get_total_song_time(connection)*(double)(value/100));
+		int change = (int)(mpd_status_get_total_song_time(connection)*(double)(value/100));
 
-		mpd_ob_player_seek(connection, change);
+		mpd_player_seek(connection, change);
 	}
 	return FALSE;
 }
@@ -573,26 +661,21 @@ int progress_seek_stop()
 /* also if volume isnt "slidable" block the user from changing it */
 int volume_change_start()
 {
-	if(!mpd_ob_check_connected(connection) || volume) return TRUE;
-	if(mpd_ob_status_get_volume(connection) == -1) return TRUE;
+	if(!mpd_check_connected(connection) || volume) return TRUE;
+	if(mpd_status_get_volume(connection) == -1) return TRUE;
 	volume = TRUE;
-	msg_push_popup("Volume: ");
 	return FALSE;
 }
 
 /* if the volume changes say it in the entry box.. this looks nice :) */    
 void volume_change_update()
 {
-	if(mpd_ob_check_connected(connection) && volume)
+	if(mpd_check_connected(connection) && volume)
 	{
 		GtkRange *scale = (GtkRange *)glade_xml_get_widget(xml_main_window, "volume_slider");
 		gdouble value = gtk_range_get_value(scale);
-		gchar *buf = g_strdup_printf(_("Volume %i%%"), (int)value);
-		msg_pop_popup();
-		msg_push_popup(buf);
-		g_free(buf);
 
-		if(mpd_ob_status_set_volume(connection,(int)value) < 0)
+		if(mpd_status_set_volume(connection,(int)value) < 0)
 		{
 			return;
 		}
@@ -601,25 +684,24 @@ void volume_change_update()
 		/* it does look ugly .. need to find a better way */
 		scroll.pos = -1;
 	}
-	else if(mpd_ob_check_connected(connection))
+	else if(mpd_check_connected(connection))
 	{
 		GtkRange *scale = (GtkRange *)glade_xml_get_widget(xml_main_window, "volume_slider");
 		gdouble value = gtk_range_get_value(scale);
-		if(value != mpd_ob_status_get_volume(connection))
+		if(value != mpd_status_get_volume(connection))
 		{
-			if(mpd_ob_status_set_volume(connection, (int)value) < 0)
+			if(mpd_status_set_volume(connection, (int)value) < 0)
 			{
 				return;
-			}                                                 			
+			}
 		}
 	}
 }
 /* apply changes and give mpd free */
 int volume_change_stop()
 {
-	msg_pop_popup();
 	volume = FALSE;
-	if(!mpd_ob_check_connected(connection)) return TRUE;
+	if(!mpd_check_connected(connection)) return TRUE;
 	return FALSE;
 }
 
@@ -639,9 +721,10 @@ void time_format_toggle()
 /* the id3 info screen */
 void id3_info()
 {
-	if(!mpd_ob_check_connected(connection)) return;
-	if(mpd_ob_player_get_current_song_id(connection) > 0);
-	call_id3_window(mpd_ob_player_get_current_song_id(connection));
+	int state = mpd_player_get_state(connection);
+	if(state == MPD_PLAYER_STOP || state == MPD_PLAYER_UNKNOWN) return;
+	if(!mpd_check_connected(connection)) return;
+	call_id3_window(mpd_player_get_current_song_id(connection));
 }
 
 void style_changed(GtkWidget *window, GtkStyle *prev, PangoLayout *lay)
@@ -666,16 +749,16 @@ int player_key_press(GtkWidget *mw, GdkEventKey *event,gpointer data)
 	/* volume up */
 	else if (event->keyval == GDK_Up)
 	{
-		mpd_ob_status_set_volume(connection,mpd_ob_status_get_volume(connection) +5);
+		mpd_status_set_volume(connection,mpd_status_get_volume(connection) +5);
 		return TRUE;
 	}
 	/* volume down */
 	else if (event->keyval == GDK_Down)
 	{
-		mpd_ob_status_set_volume(connection,mpd_ob_status_get_volume(connection) -5);
+		mpd_status_set_volume(connection,mpd_status_get_volume(connection) -5);
 		return TRUE;
 	}
-	else if (event->keyval == GDK_q && event->state == GDK_CONTROL_MASK)
+	else if (event->keyval == GDK_q && event->state&GDK_CONTROL_MASK)
 	{
 		/*gtk_main_quit();*/
 		main_quit();
@@ -692,25 +775,62 @@ int player_key_press(GtkWidget *mw, GdkEventKey *event,gpointer data)
 	}
 	return FALSE;
 }
+void player_hide()
+{
+	if(xml_main_window)
+	{
+		gtk_window_get_position(GTK_WINDOW(glade_xml_get_widget(xml_main_window, "main_window")), 
+				&player_wsize.x, &player_wsize.y);
+		gtk_widget_hide(GTK_WIDGET(glade_xml_get_widget(xml_main_window, "main_window")));
+		player_hidden = TRUE;
+	}
+}
 
 
+void player_show()
+{
+	if(xml_main_window)
+	{
+		if(player_wsize.x && player_wsize.y)
+		{
+			gtk_window_move(GTK_WINDOW(glade_xml_get_widget(xml_main_window, "main_window")),
+				player_wsize.x, player_wsize.y);
+		}
+		gtk_widget_show(GTK_WIDGET(glade_xml_get_widget(xml_main_window, "main_window")));
+		player_hidden = FALSE;
+	}
+}
+
+int player_get_hidden()
+{
+	return player_hidden;
+}
+
 /* create the player and connect signals */
-void create_player()
+void player_create()
 {
-	xml_main_window = glade_xml_new(GLADE_PATH"gmpc.glade", "main_window", NULL);
+	char *path = gmpc_get_full_glade_path("gmpc.glade");
+	xml_main_window = glade_xml_new(path, "main_window", NULL);
+	g_free(path);
+
 	/* check for errors and axit when there is no gui file */
 	if(xml_main_window == NULL)  g_error(_("Couldnt initialize GUI. Please check installation\n"));
 	glade_xml_signal_autoconnect(xml_main_window);
 
 	DISPLAY_WIDTH = glade_xml_get_widget(xml_main_window, "entry_image")->allocation.width;
 	/* set icons from the custom stock set*/
-	gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(xml_main_window, "prev_im")), "gtk-media-previous", GTK_ICON_SIZE_BUTTON);
-	gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(xml_main_window, "next_im")), "gtk-media-next", GTK_ICON_SIZE_BUTTON);	
-	gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(xml_main_window, "stop_im")), "gtk-media-stop", GTK_ICON_SIZE_BUTTON);	
-	gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(xml_main_window, "play_button_image")), "gtk-media-play", GTK_ICON_SIZE_BUTTON);	
-	gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(xml_main_window, "rand_im")), "media-random", GTK_ICON_SIZE_BUTTON);			
-	gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(xml_main_window, "rep_im")), "media-repeat", GTK_ICON_SIZE_BUTTON);			
-
+	gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(xml_main_window, "prev_im")),
+			"gtk-media-previous", GTK_ICON_SIZE_BUTTON);
+	gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(xml_main_window, "next_im")),
+			"gtk-media-next", GTK_ICON_SIZE_BUTTON);
+	gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(xml_main_window, "stop_im")),
+			"gtk-media-stop", GTK_ICON_SIZE_BUTTON);
+	gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(xml_main_window, "play_button_image")),
+			"gtk-media-play", GTK_ICON_SIZE_BUTTON);
+	gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(xml_main_window, "rand_im")),
+			"media-random", GTK_ICON_SIZE_BUTTON);
+	gtk_image_set_from_stock(GTK_IMAGE(glade_xml_get_widget(xml_main_window, "rep_im")),
+			"media-repeat", GTK_ICON_SIZE_BUTTON);
 
 	gtk_widget_set_app_paintable(glade_xml_get_widget(xml_main_window, "entry_image"),TRUE);
 	gtk_widget_set_app_paintable(glade_xml_get_widget(xml_main_window, "time_image"),TRUE);
@@ -729,12 +849,68 @@ void create_player()
 	pango_layout_set_text(time_layout, "00:00", -1);
 
 	g_signal_connect(
-			G_OBJECT(glade_xml_get_widget(xml_main_window, "main_window")),		
+			G_OBJECT(glade_xml_get_widget(xml_main_window, "main_window")),	
 			"key-press-event",
 			G_CALLBACK(player_key_press),
 			NULL);
 	/* check for errors and axit when there is no gui file */
-	gtk_timeout_add(300, (GSourceFunc)update_msg, NULL);
-	time_exposed(glade_xml_get_widget(xml_main_window, "time_image"));
-	display_exposed(glade_xml_get_widget(xml_main_window, "entry_image"));
+	gtk_timeout_add(200, (GSourceFunc)update_msg, NULL);
+	//	time_exposed(glade_xml_get_widget(xml_main_window, "time_image"));
+	//	display_exposed(glade_xml_get_widget(xml_main_window, "entry_image"));
+
+	if(cfg_get_single_value_as_int_with_default(config,"tray-icon", "enable", DEFAULT_TRAY_ICON_ENABLE) &&
+			cfg_get_single_value_as_int_with_default(config,"player", "hide-startup", DEFAULT_HIDE_ON_STARTUP))
+	{
+		player_hide();
+
+
+	}
+	else
+	{
+		player_show();
+	}
+}
+
+void player_destroy()
+{
+	if(xml_main_window)
+	{
+		GtkWidget *mw = glade_xml_get_widget(xml_main_window, "main_window");
+		g_object_unref(layout);
+		g_object_unref(time_layout);
+		if(mw)gtk_widget_destroy(mw);
+		g_object_unref(xml_main_window);
+		xml_main_window = NULL;
+		if(scroll.msg)g_free(scroll.msg);
+		if(scroll.base_msg)g_free(scroll.base_msg);
+		if(scroll.popup_msg)g_free(scroll.popup_msg);
+	}
+}
+
+void player_connection_changed(MpdObj *mi, int connect)
+{
+	if(connect){
+		msg_set_base(_("GMPC - Connected"));
+		if(cfg_get_single_value_as_int_with_default(config, "player", "window-title",TRUE))
+		{
+			if(xml_main_window != NULL)
+			{
+
+				gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget(xml_main_window, "main_window")),
+						_("Gnome Music Player Client"));
+			}
+		}                                                                                                     		
+	}
+	else
+	{
+		msg_set_base(_("gmpc - Disconnected"));
+		if(cfg_get_single_value_as_int_with_default(config, "player", "window-title",TRUE))
+		{
+			if(xml_main_window != NULL)
+			{
+				gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget(xml_main_window, "main_window")),
+						_("Gnome Music Player Client"));
+			}
+		}
+	}
 }
